/* Copyright (C) 2025 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergei Zharko [committer] */

/// \file   KfpV0Finder.h
/// \date   01.02.2025
/// \brief  A V0 finding algorithm
/// \author Sergei Zharko <s.zharko@gsi.de>

#pragma once

#include "CbmEventTriggers.h"
#include "KFParticleTopoReconstructor.h"
#include "global/RecoResults.h"

#include <memory>


namespace cbm::algo::kfp
{
  /// \class V0Finder
  /// \brief A V0-finding algorithm
  class V0Finder {
   public:
    /// \brief Default constructor
    V0Finder() = default;

    /// \brief Copy constructor
    V0Finder(const V0Finder&) = delete;

    /// \brief Move constructor
    V0Finder(V0Finder&&) = delete;

    /// \brief Destructor
    ~V0Finder() = default;

    /// \brief Copy assignment operator
    V0Finder& operator=(const V0Finder&) = delete;

    /// \brief Move assignment operator
    V0Finder& operator=(V0Finder&&) = delete;

    /// \brief  Adds particle to reconstruction list
    /// \param  pdg  A PDG code of the particle to be reconstructed
    void AddDecayToReconstructionList(int pdg) { GetKFParticleFinder()->AddDecayToReconstructionList(pdg); }

    /// \brief  Mutable access to the KfParticleFinder of the run topology reconstructor
    KFParticleFinder* GetKFParticleFinder() { return fpTopoReconstructor->GetKFParticleFinder(); }

    /// \brief  Constant access to the KfParticleFinder of the run topology reconstructor
    const KFParticleFinder* GetKFParticleFinder() const { return fpTopoReconstructor->GetKFParticleFinder(); }

    /// \brief Initializes the instance (called in the beginning of the run)
    void Init();

    /// \brief Processes a reconstructed data sample, returns a collection of fired triggers
    CbmEventTriggers Process(const RecoResults& recoEvent);

    /// \brief  Sets minimal pion DCA to primary vertex
    /// \param  dca  DCA [cm]
    void SetMinPionDca(double dca) { fMinPionDca = dca; }

    /// \brief  Sets minimal proton DCA to primary vertex
    /// \param  dca  DCA [cm]
    void SetMinProtonDca(double dca) { fMinProtonDca = dca; }

    /// \brief  Sets pion velocity range
    /// \param  vMin  Minimal velocity [cm/ns]
    /// \param  vMax  Maximal velocity [cm/ns]
    void SetPionVelocityRange(double vMin, double vMax)
    {
      fMinBetaPion = vMin / kSpeedOfLight;
      fMaxBetaPion = vMax / kSpeedOfLight;
    }

    /// \brief  Sets proton velocity range
    /// \param  vMin  Minimal velocity [cm/ns]
    /// \param  vMax  Maximal velocity [cm/ns]
    void SetProtonVelocityRange(double vMin, double vMax)
    {
      fMinBetaProton = vMin / kSpeedOfLight;
      fMaxBetaProton = vMax / kSpeedOfLight;
    }

    /// \brief  Sets the assigned PDG for primary particles
    /// \param  pdg  PDG code of the particle
    void SetPrimaryAssignedPdg(int pdg) { fPrimaryAssignedPdg = pdg; }

    /// \brief  Assignes an uncertainty to the momentum measurement
    /// \param  uncertainty  Relative uncertainty ( = sqrt(var(q/p)) / (q/p))
    void SetQpAssignedUncertainty(double uncertainty) { fQpAssignedUncertainty = uncertainty; }

    /// \brief  Sets an offset to t0
    /// \param  offset  An offset [ns]
    void SetTzeroOffset(double offset) { fTzeroOffset = offset; }

    //* Specific parameters for the KFParticleFinder
    /// \brief  Sets cut on the distance to the primary vertex from the decay vertex
    /// \param  cut  Cut value [cm]
    void SetLCut(float cut) { GetKFParticleFinder()->SetLCut(cut); }

    /// \brief  Sets cut on \f$\chi^2_{prim}\f$ of each track for 2-daughter decays
    /// \param  cut  Cut value
    void SetChiPrimaryCut2D(float cut) { GetKFParticleFinder()->SetChiPrimaryCut2D(cut); }

    /// \brief  Sets cut on \f$\chi^2_{geo}\f$ for 2-daughter decays
    /// \param  cut  Cut value
    void SetChi2Cut2D(float cut) { GetKFParticleFinder()->SetChi2Cut2D(cut); }

    /// \brief  Sets cut on \f$l/\Delta l\f$ for 2-daughter decays
    /// \param  cut  Cut value
    void SetLdLCut2D(float cut) { GetKFParticleFinder()->SetLdLCut2D(cut); }

   private:
    //* Physical constants
    static constexpr double kPionMass{0.13957039};      ///< Pion mass [GeV/c2]
    static constexpr double kProtonMass{0.938272088};   ///< Proton mass [GeV/c2]
    static constexpr double kSpeedOfLight{29.9792458};  ///< Speed of light [cm/ns]

    /// \brief An instance of the topology reconstructor
    std::unique_ptr<KFParticleTopoReconstructor> fpTopoReconstructor{std::make_unique<KFParticleTopoReconstructor>()};

    //* Different run-time cuts and flags (TODO: define in a config)
    double fTzeroOffset{0.};             ///< Offset for T0
    double fMinPionDca{1.5};             ///< Minimum DCA to PV for pions
    double fMinProtonDca{0.5};           ///< Minimum DCA to PV for protons
    double fQpAssignedUncertainty{0.1};  ///< Assigned relative uncertainty for q/p estimation
    int fPrimaryAssignedPdg{321};        ///< Assigned PDG hypothesis for primary particles
    double fMinBetaProton{0.};           ///< Minimal proton velocity (beta) [c]
    double fMaxBetaProton{1.};           ///< Maximal proton velocity (beta) [c]
    double fMinBetaPion{0.};             ///< Minimal proton velocity (beta) [c]
    double fMaxBetaPion{1.};             ///< Maximal proton velocity (beta) [c]
  };
}  // namespace cbm::algo::kfp
